<?php

namespace App\Form;

use App\Entity\Domain;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Validator\Constraints\Length;
use Symfony\Component\Validator\Constraints\NotBlank;

class DomainType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options, $CREATE = true): void
    {
        $builder
            ->add('enabled', CheckboxType::class, [
                'attr' => ['checked' => 'checked', 'value' => '1'],
            ])
            ->add('domain', TextType::class, [
                'attr' => [
                    'placeholder' => 'example.com'
                ],
                'constraints' => [
                    new NotBlank(groups: ['create']),
                    new Length(min: 1, max: 255),
                ]
            ])
            ->add('name', TextType::class, [
                'attr' => [
                    'placeholder' => "John Doe's Blog"
                ],
                'constraints' => [
                    new NotBlank(groups: ['create']),
                    new Length(min: 1, max: 255)
                ]
            ])
            ->add('defaultSortPolicy', ChoiceType::class, [
                'choices'  => Domain::SORT_POLICIES
            ])
        ;
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'data_class' => Domain::class,
        ]);
    }
}
